@@ -88,7 +88,7 @@ char *translate_cardname(char *name)
     full = malloc(strlen("default:CARD=") + strlen(name) + 1);  
     sprintf(full, "default:CARD=%s", name);
 
-                       return full;
+    return full;
 }
 
 /* Translate a card index to a ALSA cardname 
@@ -131,7 +131,8 @@ alsacard_list(PyObject *self, PyObject *args)
     {
         char name[32];
         int err;
-        
+        PyObject *item;        
+
         /* One would be tempted to think that snd_card_get_name returns a name
            that is actually meaningful for any further operations.
            
@@ -148,9 +149,10 @@ alsacard_list(PyObject *self, PyObject *args)
             Py_DECREF(result);
             return NULL;
         }
-        
-        PyList_Append(result, 
-                      PyUnicode_FromString(snd_ctl_card_info_get_id(info)));
+       
+        item = PyUnicode_FromString(snd_ctl_card_info_get_id(info));
+        PyList_Append(result, item); 
+        Py_DECREF(item);
         
         snd_ctl_close(handle);
     }
@@ -172,31 +174,29 @@ static int alsapcm_setup(alsapcm_t *self)
     snd_pcm_hw_params_t *hwparams;
     
     /* Allocate a hwparam structure on the stack, 
-       and fill it in with configuration space */
+       and fill it with configuration space */
     snd_pcm_hw_params_alloca(&hwparams);
     res = snd_pcm_hw_params_any(self->handle, hwparams);
     if (res < 0) 
         return res;
     
-    /* Fill it in with default values. */
+    /* Fill it with default values. 
+
+       We don't care if any of this fails - we'll read the actual values 
+       back out.
+     */
     snd_pcm_hw_params_any(self->handle, hwparams);
     snd_pcm_hw_params_set_access(self->handle, hwparams, 
                                  SND_PCM_ACCESS_RW_INTERLEAVED);
     snd_pcm_hw_params_set_format(self->handle, hwparams, self->format);
-    res = snd_pcm_hw_params_set_channels(self->handle, hwparams, 
-                                         self->channels);
-    if (res < 0) return res;
+    snd_pcm_hw_params_set_channels(self->handle, hwparams, 
+                                   self->channels);
 
     dir = 0;
-    res = snd_pcm_hw_params_set_rate(self->handle, hwparams, self->rate, dir);
-    if (res < 0) return res;
-    
-    res = snd_pcm_hw_params_set_period_size(self->handle, hwparams, 
+    snd_pcm_hw_params_set_rate(self->handle, hwparams, self->rate, dir);
+    snd_pcm_hw_params_set_period_size(self->handle, hwparams, 
                                       self->periodsize, dir);
-    if (res < 0) return res;
-
-    res = snd_pcm_hw_params_set_periods(self->handle, hwparams, 4, 0);
-    if (res < 0) return res;
+    snd_pcm_hw_params_set_periods(self->handle, hwparams, 4, 0);
     
     /* Write it to the device */
     res = snd_pcm_hw_params(self->handle, hwparams);
@@ -761,6 +761,59 @@ PyDoc_STRVAR(pause_doc,
 If enable is 1, playback or capture is paused. If enable is 0,\n\
 playback/capture is resumed.");
 
+static PyObject *
+alsapcm_polldescriptors(alsapcm_t *self, PyObject *args)
+{
+    int i, count, rc;
+    PyObject *result;
+    struct pollfd *fds;
+
+    if (!PyArg_ParseTuple(args,":polldescriptors"))
+        return NULL;
+
+    if (!self->handle)
+    {
+        PyErr_SetString(ALSAAudioError, "PCM device is closed");
+        return NULL;
+    }
+
+    count = snd_pcm_poll_descriptors_count(self->handle);
+    if (count < 0)
+    {
+        PyErr_SetString(ALSAAudioError,"Can't get poll descriptor count");
+        return NULL;
+    }
+
+    fds = (struct pollfd*)calloc(count, sizeof(struct pollfd));
+    if (!fds)
+    {
+        PyErr_SetString(PyExc_MemoryError, "Out of memory");
+        return NULL;
+    }
+
+    result = PyList_New(count);
+    rc = snd_pcm_poll_descriptors(self->handle, fds, (unsigned int)count);
+    if (rc != count)
+    {
+        PyErr_SetString(ALSAAudioError,"Can't get poll descriptors");
+        return NULL;
+    }
+
+    for (i = 0; i < count; ++i)
+    {
+        PyList_SetItem(result, i,
+                       Py_BuildValue("II", fds[i].fd, fds[i].events));
+    }
+
+    return result;
+}
+
+PyDoc_STRVAR(pcm_polldescriptors_doc,
+"polldescriptors() -> List of tuples (fd, eventmask).\n\
+\n\
+Return a list of file descriptors and event masks\n\
+suitable for use with poll.");
+
 
 /* ALSA PCM Object Bureaucracy */
 
@@ -779,6 +832,8 @@ static PyMethodDef alsapcm_methods[] = {
     {"write", (PyCFunction)alsapcm_write, METH_VARARGS, write_doc},
     {"pause", (PyCFunction)alsapcm_pause, METH_VARARGS, pause_doc},
     {"close", (PyCFunction)alsapcm_close, METH_VARARGS, pcm_close_doc},
+    {"polldescriptors", (PyCFunction)alsapcm_polldescriptors, METH_VARARGS,
+     pcm_polldescriptors_doc},
     {NULL, NULL}
 };
 
@@ -1152,6 +1207,8 @@ static PyObject *
 alsamixer_volumecap(alsamixer_t *self, PyObject *args) 
 {
     PyObject *result;
+    PyObject *item;
+
     if (!PyArg_ParseTuple(args,":volumecap")) return NULL;
 
     if (!self->handle) 
@@ -1162,18 +1219,42 @@ alsamixer_volumecap(alsamixer_t *self, PyObject *args)
 
     result = PyList_New(0);
     if (self->volume_cap&MIXER_CAP_VOLUME) 
-        PyList_Append(result,PyUnicode_FromString("Volume"));
+    {
+        item = PyUnicode_FromString("Volume");
+        PyList_Append(result, item);
+        Py_DECREF(item);
+    }
     if (self->volume_cap&MIXER_CAP_VOLUME_JOINED) 
-        PyList_Append(result,PyUnicode_FromString("Joined Volume"));
+    {
+        item = PyUnicode_FromString("Joined Volume"); 
+        PyList_Append(result, item);
+        Py_DECREF(item);
+    }
     if (self->volume_cap&MIXER_CAP_PVOLUME) 
-        PyList_Append(result,PyUnicode_FromString("Playback Volume"));
+    {
+        item = PyUnicode_FromString("Playback Volume");
+        PyList_Append(result, item);
+        Py_DECREF(item);
+    }
     if (self->volume_cap&MIXER_CAP_PVOLUME_JOINED) 
-        PyList_Append(result,PyUnicode_FromString("Joined Playback Volume"));
+    {
+        item = PyUnicode_FromString("Joined Playback Volume");
+        PyList_Append(result, item);
+        Py_DECREF(item);
+    }
     if (self->volume_cap&MIXER_CAP_CVOLUME) 
-        PyList_Append(result,PyUnicode_FromString("Capture Volume"));
+    {
+        item = PyUnicode_FromString("Capture Volume");
+        PyList_Append(result, item);
+        Py_DECREF(item);
+    }
     if (self->volume_cap&MIXER_CAP_CVOLUME_JOINED) 
-        PyList_Append(result,PyUnicode_FromString("Joined Capture Volume"));
-    
+    {
+        item = PyUnicode_FromString("Joined Capture Volume");
+        PyList_Append(result, item);
+        Py_DECREF(item);
+    }
+
     return result;
 }
 
@@ -1194,6 +1275,8 @@ static PyObject *
 alsamixer_switchcap(alsamixer_t *self, PyObject *args) 
 {
     PyObject *result;
+    PyObject *item;
+
     if (!PyArg_ParseTuple(args,":switchcap")) 
         return NULL;
 
@@ -1205,19 +1288,47 @@ alsamixer_switchcap(alsamixer_t *self, PyObject *args)
 
     result = PyList_New(0);
     if (self->volume_cap&MIXER_CAP_SWITCH) 
-        PyList_Append(result,PyUnicode_FromString("Mute"));
+    {
+        item = PyUnicode_FromString("Mute");
+        PyList_Append(result, item);
+        Py_DECREF(item);
+    }
     if (self->volume_cap&MIXER_CAP_SWITCH_JOINED) 
-        PyList_Append(result,PyUnicode_FromString("Joined Mute"));
+    {
+        item = PyUnicode_FromString("Joined Mute");
+        PyList_Append(result, item);
+        Py_DECREF(item);
+    }
     if (self->volume_cap&MIXER_CAP_PSWITCH) 
-        PyList_Append(result,PyUnicode_FromString("Playback Mute"));
+    {
+        item = PyUnicode_FromString("Playback Mute");
+        PyList_Append(result, item);
+        Py_DECREF(item);
+    }
     if (self->volume_cap&MIXER_CAP_PSWITCH_JOINED) 
-        PyList_Append(result,PyUnicode_FromString("Joined Playback Mute"));
+    {
+        item = PyUnicode_FromString("Joined Playback Mute");
+        PyList_Append(result, item);
+        Py_DECREF(item);
+    }
     if (self->volume_cap&MIXER_CAP_CSWITCH) 
-        PyList_Append(result,PyUnicode_FromString("Capture Mute"));
+    {
+        item = PyUnicode_FromString("Capture Mute");
+        PyList_Append(result, item);
+        Py_DECREF(item);
+    }
     if (self->volume_cap&MIXER_CAP_CSWITCH_JOINED) 
-        PyList_Append(result,PyUnicode_FromString("Joined Capture Mute"));
+    {
+        item = PyUnicode_FromString("Joined Capture Mute");
+        PyList_Append(result, item);
+        Py_DECREF(item);
+    }
     if (self->volume_cap&MIXER_CAP_CSWITCH_EXCLUSIVE) 
-        PyList_Append(result,PyUnicode_FromString("Capture Exclusive"));
+    {
+        item = PyUnicode_FromString("Capture Exclusive");
+        PyList_Append(result, item);
+        Py_DECREF(item);
+    }
 
     return result;
 }
@@ -1273,6 +1384,7 @@ alsamixer_getvolume(alsamixer_t *self, PyObject *args)
     long ival;
     char *dirstr = 0;
     PyObject *result;
+    PyObject *item;
     
     if (!PyArg_ParseTuple(args,"|s:getvolume",&dirstr)) 
         return NULL;
@@ -1309,19 +1421,21 @@ alsamixer_getvolume(alsamixer_t *self, PyObject *args)
             snd_mixer_selem_has_playback_channel(elem, channel)) 
         {
             snd_mixer_selem_get_playback_volume(elem, channel, &ival);
-            PyList_Append(
-                result, PyLong_FromLong(alsamixer_getpercentage(self->pmin, 
-                                                                self->pmax, 
-                                                                ival)));
+            item = PyLong_FromLong(alsamixer_getpercentage(self->pmin,
+                                                           self->pmax,
+                                                           ival));
+            PyList_Append(result, item);
+            Py_DECREF(item);
         }
         else if (direction == 1
                  && snd_mixer_selem_has_capture_channel(elem, channel)
                  && snd_mixer_selem_has_capture_volume(elem)) {
             snd_mixer_selem_get_capture_volume(elem, channel, &ival);
-            PyList_Append(
-                result, PyLong_FromLong(alsamixer_getpercentage(self->cmin,
-                                                                self->cmax, 
-                                                                ival)));
+            item = PyLong_FromLong(alsamixer_getpercentage(self->cmin,
+                                                           self->cmax,
+                                                           ival));
+            PyList_Append(result, item);
+            Py_DECREF(item);
         }
     }
 
@@ -1502,6 +1616,8 @@ alsamixer_getmute(alsamixer_t *self, PyObject *args)
     int i;
     int ival;
     PyObject *result;
+    PyObject *item;
+
     if (!PyArg_ParseTuple(args,":getmute")) 
         return NULL;
 
@@ -1517,13 +1633,18 @@ alsamixer_getmute(alsamixer_t *self, PyObject *args)
         PyErr_SetString(ALSAAudioError,"Mixer has no mute switch");
         return NULL;
     }    
+
     result = PyList_New(0);
+
     for (i = 0; i <= SND_MIXER_SCHN_LAST; i++) 
     {
         if (snd_mixer_selem_has_playback_channel(elem, i)) 
         {
             snd_mixer_selem_get_playback_switch(elem, i, &ival);
-            PyList_Append(result,PyLong_FromLong(!ival));
+
+            item = PyLong_FromLong(!ival);
+            PyList_Append(result, item);
+            Py_DECREF(item);
         }
     }
     return result;
@@ -1545,6 +1666,8 @@ alsamixer_getrec(alsamixer_t *self, PyObject *args)
     int i;
     int ival;
     PyObject *result;
+    PyObject *item;
+
     if (!PyArg_ParseTuple(args,":getrec")) 
         return NULL;
     
@@ -1560,13 +1683,17 @@ alsamixer_getrec(alsamixer_t *self, PyObject *args)
         PyErr_SetString(ALSAAudioError,"Mixer has no record switch");
         return NULL;
     }    
+
     result = PyList_New(0);
+
     for (i = 0; i <= SND_MIXER_SCHN_LAST; i++) 
     {
         if (snd_mixer_selem_has_capture_channel(elem, i)) 
         {
             snd_mixer_selem_get_capture_switch(elem, i, &ival);
-            PyList_Append(result,PyLong_FromLong(ival));
+            item = PyLong_FromLong(ival);
+            PyList_Append(result, item);
+            Py_DECREF(item);
         }
     }
     return result;
@@ -1784,6 +1911,58 @@ If omitted, the capture flag is set for all channels.\n\
 \n\
 This method will fail if the mixer has no capture switch capabilities");
 
+static PyObject *
+alsamixer_polldescriptors(alsamixer_t *self, PyObject *args) 
+{
+    int i, count, rc;
+    PyObject *result;
+    struct pollfd *fds;
+
+    if (!PyArg_ParseTuple(args,":polldescriptors")) 
+        return NULL;
+    
+    if (!self->handle) 
+    {
+        PyErr_SetString(ALSAAudioError, "Mixer is closed");
+        return NULL;
+    }
+    
+    count = snd_mixer_poll_descriptors_count(self->handle);
+    if (count < 0)
+    {
+        PyErr_SetString(ALSAAudioError,"Can't get poll descriptor count");
+        return NULL;
+    }
+
+    fds = (struct pollfd*)calloc(count, sizeof(struct pollfd));
+    if (!fds)
+    {
+        PyErr_SetString(PyExc_MemoryError, "Out of memory");
+        return NULL;
+    }
+
+    result = PyList_New(count);
+    rc = snd_mixer_poll_descriptors(self->handle, fds, (unsigned int)count);
+    if (rc != count)
+    {
+        PyErr_SetString(ALSAAudioError,"Can't get poll descriptors");
+        return NULL;
+    }
+
+    for (i = 0; i < count; ++i)
+    {
+        PyList_SetItem(result, i, 
+                       Py_BuildValue("II", fds[i].fd, fds[i].events));
+    }
+
+    return result;
+}
+
+PyDoc_STRVAR(polldescriptors_doc,
+"polldescriptors() -> List of tuples (fd, eventmask).\n\
+\n\
+Return a list of file descriptors and event masks\n\
+suitable for use with poll to monitor changes on this mixer.");
 
 static PyMethodDef alsamixer_methods[] = {
     {"cardname", (PyCFunction)alsamixer_cardname, METH_VARARGS, 
@@ -1805,6 +1984,9 @@ static PyMethodDef alsamixer_methods[] = {
      setvolume_doc},
     {"setmute", (PyCFunction)alsamixer_setmute, METH_VARARGS, setmute_doc},
     {"setrec", (PyCFunction)alsamixer_setrec, METH_VARARGS, setrec_doc},
+    {"polldescriptors", (PyCFunction)alsamixer_polldescriptors, METH_VARARGS, 
+     polldescriptors_doc},
+    
     {NULL, NULL}
 };
 
