@@ -4,8 +4,7 @@
 
 
 # Footnote: I'd normally use print instead of sys.std(out|err).write,
-# but we're in the middle of the conversion between python 2 and 3
-# and this code runs on both versions without conversion
+# but this version runs on python 2 and python 3 without conversion
 
 import sys
 import wave
@@ -14,13 +13,17 @@ import alsaaudio
 
 def play(device, f):    
 
+
+    sys.stdout.write('%d channels, %d sampling rate\n' % (f.getnchannels(),
+                                                          f.getframerate()))
     # Set attributes
     device.setchannels(f.getnchannels())
     device.setrate(f.getframerate())
 
-    # We assume signed data, little endian
+    # 8bit is unsigned in wav files
     if f.getsampwidth() == 1:
-        device.setformat(alsaaudio.PCM_FORMAT_S8)
+        device.setformat(alsaaudio.PCM_FORMAT_U8)
+    # Otherwise we assume signed data, little endian
     elif f.getsampwidth() == 2:
         device.setformat(alsaaudio.PCM_FORMAT_S16_LE)
     elif f.getsampwidth() == 3:
@@ -29,6 +32,8 @@ def play(device, f):
         device.setformat(alsaaudio.PCM_FORMAT_S32_LE)
     else:
         raise ValueError('Unsupported format')
+
+    device.setperiodsize(320)
     
     data = f.readframes(320)
     while data:
